package es.ucm.fdi.grasia.faerie.examples.bottomUp.abs;

import java.net.URI;
import java.util.Map;
import java.util.Properties;

import org.apache.felix.scr.annotations.Component;

import es.ucm.fdi.grasia.faerie.core.AbstractAttributeUpdaterComponent;
import es.ucm.fdi.grasia.faerie.core.Attribute;
import es.ucm.fdi.grasia.faerie.core.ContextElement;
import es.ucm.fdi.grasia.faerie.core.ContextEvent;
import es.ucm.fdi.grasia.faerie.core.ContextValue;
import es.ucm.fdi.grasia.faerie.core.ContextObserver;
import es.ucm.fdi.grasia.faerie.core.model.BasicContextValue;
import es.ucm.fdi.grasia.faerie.core.util.ContextURIs;
import es.ucm.fdi.grasia.faerie.core.util.Functions;
import es.ucm.fdi.grasia.faerie.examples.bottomUp.model.Attribute1;
import es.ucm.fdi.grasia.faerie.examples.bottomUp.model.Attribute2;
import es.ucm.fdi.grasia.faerie.examples.bottomUp.model.Sensor;

@Component(inherit = true)
@SuppressWarnings({ "unchecked", "rawtypes" })
public class BottomUpEvalAbs extends AbstractAttributeUpdaterComponent {

	private static Properties properties = Functions.obtainProperties("teacherFinding.properties");
	private Sensor s1 = null;
	private Sensor s2 = null;

	@Override
	public void addMyReferencedEntities() {
		s1 = this.addReferencedEntity(new Sensor(URI.create(properties.getProperty("location.url")), URI.create(
				"/example/"), URI.create("s1")));
		s2 = this.addReferencedEntity(
				new Sensor(ContextURIs.localAuthority(), URI.create("/example/"), URI.create("s2")));
	}

	@Override
	public int handle(ContextEvent event) {
		return Functions.contextElementAddedOrRemoved(Attribute1.class.getName(), event) ? ContextObserver.HANDLE_WRITE
				: ContextObserver.HANDLE_NONE;
	}

	@Override
	public void elementAdded(Attribute attributeInstance, ContextEvent event) {
		this.putUpdatedAttributeInstance(attributeInstance);

		URI attInstanceId = attributeInstance.getId();

		Attribute2 att2s1 = new Attribute2(s1);
		att2s1.setRemotelyUpdateable(true);
		this.addAttributeInstanceLinkedElement(attInstanceId, att2s1);

		Attribute2 att2s2 = new Attribute2(s2);
		this.addAttributeInstanceLinkedElement(attInstanceId, att2s2);

		this.updateAttributeInstance(attInstanceId);
	}

	@Override
	public synchronized void attributeInstanceLinkedElementsUpdated(Attribute attributeInstance,
			Map<URI, ContextElement> linkedElements) {
		Attribute2 s1Att2 = (Attribute2) linkedElements.get(ContextURIs.attributeInstanceURI(Attribute2.class,
				s1.getId()));
		Attribute2 s2Att2 = (Attribute2) linkedElements.get(ContextURIs.attributeInstanceURI(Attribute2.class,
				s2.getId()));

		ContextValue v1 = s1Att2.getValue();
		ContextValue v2 = s2Att2.getValue();

		Object data1 = (v1 != null) ? v1.getData() : null;
		Object data2 = (v2 != null) ? v2.getData() : null;

		attributeInstance.update(new BasicContextValue("Hello " + data1 + " & " + data2 + "!", 1.0,
				this.getBundleEntity()));
	}
}
